import sys
import time 
from PhidgetHelperFunctions import *
from Phidget22.Devices.TemperatureSensor import *
from Phidget22.PhidgetException import *
from Phidget22.Phidget import *
from Phidget22.Net import *

"""
* Configures the device's DataInterval and ChangeTrigger.
* Displays info about the attached phidget channel.
* Fired when a Phidget channel with onAttachHandler registered attaches
*
* @param self The Phidget channel that fired the attach event
"""
def onAttachHandler(self):
    
    ph = self

    """
    * Set the DataInterval inside of the attach handler to initialize the device with this value.
    * DataInterval defines the minimum time between TemperatureChange events.
    * DataInterval can be set to any value from MinDataInterval to MaxDataInterval.
    """
    print("\tSetting DataInterval to 1000ms")
    try:
        ph.setDataInterval(1000)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Setting DataInterval: \n\t")
        DisplayError(e)
        return

    """
    * Set the TemperatureChangeTrigger inside of the attach handler to initialize the device with this value.
    * TemperatureChangeTrigger will affect the frequency of TemperatureChange events, by limiting them to only occur when
    * the temperature changes by at least the value set.
    """
    print("\tSetting Temperature ChangeTrigger to 0.0")
    try:
        ph.setTemperatureChangeTrigger(0.0)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Set TemperatureChangeTrigger: \n\t")
        DisplayError(e)
        return
        
    try:
        serialNumber = ph.getDeviceSerialNumber()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get DeviceSerialNumber: \n\t")
        DisplayError(e)
        return

    try:
        channel = ph.getChannel()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get Channel: \n\t")
        DisplayError(e)
        return

    # Check if this is a VINT device
    try:
        hub = ph.getHub()
    except PhidgetException as e:
        if(e.code == ErrorCode.EPHIDGET_WRONGDEVICE):
            print("\nAttach Event:\n\t-> Serial Number: " + str(serialNumber) +
                    "\n\t-> Channel " + str(channel) + "\n")
        return
    try:
        hubPort = ph.getHubPort()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get HubPort: \n\t")
        DisplayError(e)
        return

    print("\nAttach Event:\n\t-> Serial Number: " + str(serialNumber) +
          "\n\t-> Hub Port: " + str(hubPort) + "\n\t-> Channel " + str(channel) + "\n")
    return

"""
* Displays info about the detached phidget channel.
* Fired when a Phidget channel with onDetachHandler registered detaches
*
* @param self The Phidget channel that fired the attach event
"""
def onDetachHandler(self):

    ph = self

    try:
        serialNumber = ph.getDeviceSerialNumber()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get DeviceSerialNumber: \n\t")
        DisplayError(e)
        return

    try:
        channel = ph.getChannel()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get Channel: \n\t")
        DisplayError(e)
        return

     # Check if this is a VINT device
    try:
        hub = ph.getHub()
    except PhidgetException as e:
        if(e.code == ErrorCode.EPHIDGET_WRONGDEVICE):
            print("\nDetach Event:\n\t-> Serial Number: " + str(serialNumber) +
                    "\n\t-> Channel " + str(channel) + "\n")
        return

    try:
        hubPort = ph.getHubPort()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get HubPort: \n\t")
        DisplayError(e)
        return

    print("\nDetach Event:\n\t-> Serial Number: " + str(serialNumber) +
          "\n\t-> Hub Port: " + str(hubPort) + "\n\t-> Channel " + str(channel) + "\n")
    return

"""
* Writes phidget error info to stderr.
* Fired when a Phidget channel with onErrorHandler registered encounters an error in the library
*
* @param self The Phidget channel that fired the attach event
* @param errorCode the code associated with the error of enum type ph.ErrorEventCode
* @param errorString string containing the description of the error fired
"""
def onErrorHandler(self, errorCode, errorString):

    sys.stderr.write("[Phidget Error Event] -> " + errorString + " (" + str(errorCode) + ")\n")

"""
* Sets the event handlers for Phidget Attach, Phidget Detach, Phidget Error events
*
* @param ph The Phidget channel to add event handlers to
* @return if the operation succeeds
* @raise PhidgetException if it fails
"""
def SetAttachDetachError_Handlers(ph):
    print("\n--------------------------------------")
    print("\nSetting OnAttachHandler...")
    try:
        ph.setOnAttachHandler(onAttachHandler)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Set Attach Handler: \n\t")
        DisplayError(e)
        raise

    print("Setting OnDetachHandler...")
    try:
        ph.setOnDetachHandler(onDetachHandler)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Set Detach Handler: \n\t")
        DisplayError(e)
        raise

    print("Setting OnErrorHandler...")
    try:
        ph.setOnErrorHandler(onErrorHandler)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Set Error Handler: \n\t")
        DisplayError(e)
        raise

    return

"""
* Outputs the TemperatureSensor's most recently reported temperature.
* Fired when a TemperatureSensor channel with onTemperatureChangeHandler registered meets DataInterval and ChangeTrigger criteria
*
* @param self The TemperatureSensor channel that fired the TemperatureChange event
* @param temperature The reported temperature from the TemperatureSensor channel
"""
def onTemperatureChangeHandler(self, temperature):

    print("[Temperature Event] -> Temperature: " + str(temperature))
    
"""
* Creates a new instance of a TemperatureSensor channel.
*
* @param pvih Pointer to the PhidgetTemperatureSensorHandle channel to create
* @return if the operation succeeds
* @raise PhidgetException if it fails
"""
def CreateTemperatureSensor(pvih):

    print("Creating TemperatureSensor Channel...")
    try:
        pvih.create()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Creating TemperatureSensor: \n\t")
        DisplayError(e)
        raise

    return

"""
* Sets the event handler for TemperatureSensor's TemperatureChange event
*
* @param pvih The PhidgetTemperatureSensorHandle channel to add the event to
* @param fptr The callback function to be called when a TemperatureChange event is fired
* @return if the operation succeeds
* @raise PhidgetException if it fails
"""
def SetTemperatureHandler(pvih, fptr):

    if (not (fptr is None)):
        print("\n--------------------\n"
            "\n  | Temperature change events contain the most recent temperature received from the device.\n"
            "  | The linked TemperatureChange function will run as an event at every DataInterval.\n"
            "  | These events will not occur until a change in temperature >= to the set ChangeTrigger has occurred.\n"
            "  | DataInterval and ChangeTrigger should initially be set in the device AttachHandler function.")
    
    print("")
    if (fptr is None):
        print("Clearing OnTemperatureChangeHandler...")
    else:
        print("Setting OnTemperatureChangeHandler...")

    print("\n--------------------")
    try:
        pvih.setOnTemperatureChangeHandler(fptr)
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Setting TemperatureChangeHandler: \n\t")
        DisplayError(e)
        raise
    
    return
    

"""
* Creates, configures, and opens a TemperatureSensor channel.
* Displays Temperature events for 10 seconds
* Closes out TemperatureSensor channel
*
* @return 0 if the program exits successfully, 1 if it exits with errors.
"""
def main():
    try:
        """
        * Allocate a new Phidget Channel object
        """
        try:
            ch = TemperatureSensor()
        except PhidgetException as e:
            sys.stderr.write("Runtime Error -> Creating TemperatureSensor: \n\t")
            DisplayError(e)
            raise
        except RuntimeError as e:
            sys.stderr.write("Runtime Error -> Creating TemperatureSensor: \n\t" + e)
            raise

        """
        * Set matching parameters to specify which channel to open
        """
        SetSerialNumber(ch)
        
        SetVINTProperties(ch)
        
        SetChannel(ch)
        
        SetupNetwork(ch)
        
        """
        * Add event handlers before calling open so that no events are missed.
        """
        SetAttachDetachError_Handlers(ch)
        
        SetTemperatureHandler(ch, onTemperatureChangeHandler)
        
        """
        * Open the channel with a timeout
        """
        OpenPhidgetChannel_waitForAttach(ch, 5000)
        
        print("Sampling data for 10 seconds...")

        time.sleep(10)

        """
         * Perform clean up and exit
         """

        SetTemperatureHandler(ch, None)
        
        print("\nDone Sampling...")

        print("Cleaning up...")
        ClosePhidgetChannel(ch)
        print("\nExiting...")
        print("Press ENTER to end program.")
        readin = sys.stdin.readline(1)
        return 0

    except PhidgetException as e:
        sys.stderr.write("\nExiting with error(s)...")
        DisplayError(e)
        print("Press ENTER to end program.")
        readin = sys.stdin.readline(1)
        return 1

main()

